/* ez-blob.h: Serialising description and utils.

   Copyright (C) 2019 Michael Zucchi

   This program is free software: you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation, either version 3 of
   the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this program. If not, see
   <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <stdint.h>
#include <string.h>

#include <assert.h>

#include "ez-blob.h"
#include "ez-list.h"

void ez_blob_free_raw(const ez_blob_desc *desc, void *p) {
	for (int i=0,dlen = desc->bd_length;i<dlen;i++) {
		const ez_blob_desc *d = &desc[i+1];
		void *v = p + d->bd_offset;
		
		switch (d->bd_type) {
		case EZ_BLOB_STRING:
			free(((char **)v)[0]);
			break;
		case EZ_BLOB_BLOB:
			free(((struct ez_blob *)v)->data);
			break;
		case EZ_BLOB_STRUCT:
			ez_blob_free_raw(d->bd_table, v);
			break;
		case EZ_BLOB_POINTER:
			ez_blob_free(d->bd_table, ((void **)v)[0]);
			break;
		case EZ_BLOB_LIST: {
			ez_list *l = (ez_list *)v;
			for (ez_node *w = ez_list_head(l), *n = ez_node_succ(w);
			     n;
			     w = n, n=ez_node_succ(n)) {
				ez_blob_free(d->bd_table, w);
			}
			break;
		}
		default:
			break;
		}
	}
}

void ez_blob_free(const ez_blob_desc *d, void *p) {
	if (p) {
		ez_blob_free_raw(d, p);
		free(p);
	}
}

void ez_blob_init(const ez_blob_desc *desc, void *p) {
	memset(p, 0, desc->bd_offset);

	for (int i=0,dlen = desc->bd_length;i<dlen;i++) {
		const ez_blob_desc *d = &desc[i+1];

		if (d->bd_type == EZ_BLOB_LIST)
			ez_list_init((ez_list *)(p + d->bd_offset));
	}
}

void *ez_blob_alloc(const ez_blob_desc *d) {
	void *p = malloc(d->bd_offset);
	if (p)
		ez_blob_init(d, p);
	return p;
}
