

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "ez-set.h"

struct entry {
	ez_node en;
	char *value;
};

static unsigned int entry_hash(const void *d) {
	const struct entry *e = d;

	return ez_hash_string(e->value);
}

static int entry_equals(const void *d, const void *e) {
	const struct entry *a = d;
	const struct entry *b = e;

	return strcmp(a->value, b->value) == 0;
}

static void entry_free(void *d) {
	struct entry *a = d;

	free(a->value);
	free(a);
}

int main(int argc, char **argv) {
	FILE *fp = fopen("/usr/share/dict/words", "r");
	char word[256];
	ez_set *set;
	int count = 0;

	set = ez_set_new(entry_hash, entry_equals, entry_free);
	
	while (fgets(word, 256, fp)) {
		size_t len = strlen(word);
		if (len > 0) {
			struct entry *e = malloc(sizeof(*e));

			word[len-1] = 0;
			e->value = strdup(word);

			e = ez_set_put(set, e);
			if (e) {
				printf("duplicate! %s\n", e->value);
			}
			count += 1;
		}
	}
	fclose(fp);

	// calc some stats
	int min = 1000, max = 0;
	int n = 0, t = 0;
	int hist[64] = { 0 };
	for (int i=0;i<=set->mask;i++) {
		int c = 0;
		for (ez_node *e = set->table[i];e;e=ez_node_next(e))
			c++;
		min = c < min ? c : min;
		max = c > max ? c : max;
		if (c != 0) {
			n += 1;
			t += c;
		}
		if (0 && c > 4) {
			printf(" %5d:", i);
			for (ez_node *e = set->table[i];e;e=ez_node_next(e)) {
				char *value = ((struct entry *)e)->value;
				printf(" %08x '%s'", set->hash(e), value);
			}
			printf("\n");
		}
		hist[c < 64 ? c : 63] += 1;
	}
	printf("chains %d length %d - %d  ave %f (> 0)\n", set->mask+1, min, max, (float)t / n);
	max = max < 64 ? max : 63;
	for (int i=0;i<=max;i++) {
		printf(" %5d: %d\n", i, hist[i]);
	}
	printf("\n");

	{
		struct entry *e;
		struct entry key = {
			.value = "Abner"
		};
		e = ez_set_get(set, &key);
		printf("lookup %s = %p %s\n", key.value, e, e->value);
		key.value = "fuckwit";
		e = ez_set_get(set, &key);
		printf("lookup %s = %p %s\n", key.value, e, e ? e->value : "");
	}
	
	ez_set_scan scan;

	// iterate
	int ncount = 0;
	for (struct entry *n = ez_set_scan_init(set, &scan); n ; n = ez_set_scan_next(&scan)) {
		ncount ++;
	}
	printf("size %d insert count %d scan count %d\n", ez_set_size(set), count, ncount);

	printf("set_clear\n");
	ez_set_clear(set);
	printf(" size %d\n", ez_set_size(set));
#if 0
	
	// iterate and clear all
	struct entry *e;
	e = ez_set_scan_init(set, &scan);
	int rcount = 0;
	while (e) {
		struct entry *p = e;

		e = ez_set_scan_remove(&scan);
		rcount ++;

		//printf(" %s\n", p->value);
		
		free(p->value);
		free(p);
	}
	printf("size %d rcount %d\n", ez_set_size(set), rcount);
#endif

	ez_set_free(set);
}

